import express from "express";
import PermissionHelper from "../../helpers/permissionHelper";

import award from "./settings/award";
import communicationType from "./settings/communicationType";
import executivePosition from "./settings/executivePosition";
import membershipStatus from "./settings/membershipStatus";
import qualification from "./settings/qualification";
import calendarType from "./settings/calendarType";
import queryStore from "./settings/queryStore";
import template from "./settings/template";
import templateUsage from "./settings/templateUsage";
import newsletterConfig from "./settings/newsletterConfig";

import member from "./club/member";
import protocol from "./club/protocol";
import calendar from "./club/calendar";
import queryBuilder from "./club/queryBuilder";
import newsletter from "./club/newsletter";

import role from "./user/role";
import user from "./user/user";
import invite from "./user/invite";
import api from "./user/webapi";
import preventWebapiAccess from "../../middleware/preventWebApiAccess";

var router = express.Router({ mergeParams: true });

router.use("/award", PermissionHelper.passCheckMiddleware("read", "settings", "award"), award);
router.use(
  "/communicationtype",
  PermissionHelper.passCheckMiddleware("read", "settings", "communication_type"),
  communicationType
);
router.use(
  "/executiveposition",
  PermissionHelper.passCheckMiddleware("read", "settings", "executive_position"),
  executivePosition
);
router.use(
  "/membershipstatus",
  PermissionHelper.passCheckMiddleware("read", "settings", "membership_status"),
  membershipStatus
);
router.use("/qualification", PermissionHelper.passCheckMiddleware("read", "settings", "qualification"), qualification);
router.use("/calendartype", PermissionHelper.passCheckMiddleware("read", "settings", "calendar_type"), calendarType);
router.use("/querystore", PermissionHelper.passCheckMiddleware("read", "settings", "query_store"), queryStore);
router.use("/template", PermissionHelper.passCheckMiddleware("read", "settings", "template"), template);
router.use("/templateusage", PermissionHelper.passCheckMiddleware("read", "settings", "template_usage"), templateUsage);
router.use(
  "/newsletterconfig",
  PermissionHelper.passCheckMiddleware("read", "settings", "newsletter_config"),
  newsletterConfig
);

router.use("/member", PermissionHelper.passCheckMiddleware("read", "club", "member"), member);
router.use("/protocol", PermissionHelper.passCheckMiddleware("read", "club", "protocol"), protocol);
router.use("/calendar", PermissionHelper.passCheckMiddleware("read", "club", "calendar"), calendar);
router.use("/querybuilder", PermissionHelper.passCheckMiddleware("read", "club", "query"), queryBuilder);
router.use("/newsletter", PermissionHelper.passCheckMiddleware("read", "club", "newsletter"), newsletter);

router.use("/role", PermissionHelper.passCheckMiddleware("read", "user", "role"), role);
router.use("/user", PermissionHelper.passCheckMiddleware("read", "user", "user"), user);
router.use("/invite", PermissionHelper.passCheckMiddleware("read", "user", "user"), invite);
router.use("/webapi", preventWebapiAccess, PermissionHelper.passCheckMiddleware("read", "user", "webapi"), api);

export default router;
